@@ -16,17 +16,37 @@ try:
 except ImportError:  # pragma: no cover.
     yaml = None  # pragma: no cover.
 try:
-    import toml
+    if sys.version_info >= (3, 11):
+        import tomllib as tomli
+    else:
+        import tomli
 except ImportError:  # pragma: no cover.
-    toml = None  # pragma: no cover.
+    tomli = None  # pragma: no cover.
+try:
+    import tomli_w
+except ImportError:  # pragma: no cover.
+    tomli_w = None  # pragma: no cover.
 try:
     import clevercsv
+    csv = None
 except ImportError:  # pragma: no cover.
+    import csv
     clevercsv = None  # pragma: no cover.
+try:
+    import orjson
+except ImportError:  # pragma: no cover.
+    orjson = None
+try:
+    from pydantic import BaseModel as PydanticBaseModel
+except ImportError:  # pragma: no cover.
+    PydanticBaseModel = None
+
 from copy import deepcopy
 from functools import partial
 from collections.abc import Mapping
-from deepdiff.helper import (strings, get_type, TEXT_VIEW)
+from deepdiff.helper import (
+    strings, get_type, TEXT_VIEW, np_float32, np_float64, np_int32, np_int64
+)
 from deepdiff.model import DeltaResult
 
 logger = logging.getLogger(__name__)
@@ -179,7 +199,7 @@ class SerializationMixin:
             '{"type_changes": {"root": {"old_type": "A", "new_type": "B", "old_value": "obj A", "new_value": "obj B"}}}'
         """
         dic = self.to_dict(view_override=TEXT_VIEW)
-        return json.dumps(dic, default=json_convertor_default(default_mapping=default_mapping), **kwargs)
+        return json_dumps(dic, default_mapping=default_mapping, **kwargs)
 
     def to_dict(self, view_override=None):
         """
@@ -195,7 +215,7 @@ class SerializationMixin:
         view = view_override if view_override else self.view
         return dict(self._get_view_results(view))
 
-    def _to_delta_dict(self, directed=True, report_repetition_required=True):
+    def _to_delta_dict(self, directed=True, report_repetition_required=True, always_include_values=False):
         """
         Dump to a dictionary suitable for delta usage.
         Unlike to_dict, this is not dependent on the original view that the user chose to create the diff.
@@ -221,7 +241,7 @@ class SerializationMixin:
         if self.group_by is not None:
             raise ValueError(DELTA_ERROR_WHEN_GROUP_BY)
 
-        result = DeltaResult(tree_results=self.tree, ignore_order=self.ignore_order)
+        result = DeltaResult(tree_results=self.tree, ignore_order=self.ignore_order, always_include_values=always_include_values)
         result.remove_empty_keys()
         if report_repetition_required and self.ignore_order and not self.report_repetition:
             raise ValueError(DELTA_IGNORE_ORDER_NEEDS_REPETITION_REPORT)
@@ -236,6 +256,9 @@ class SerializationMixin:
             # and will be omitted when counting distance. (Look inside the distance module.)
             result['_numpy_paths'] = self._numpy_paths
 
+        if self.iterable_compare_func:
+            result['_iterable_compare_func_was_used'] = True
+
         return deepcopy(dict(result))
 
     def pretty(self):
@@ -300,7 +323,7 @@ class _RestrictedPickler(pickle.Pickler):
         return None
 
 
-def pickle_dump(obj, file_obj=None):
+def pickle_dump(obj, file_obj=None, protocol=4):
     """
     **pickle_dump**
     Dumps the obj into pickled content.
@@ -318,21 +341,21 @@ def pickle_dump(obj, file_obj=None):
     """
     file_obj_passed = bool(file_obj)
     file_obj = file_obj or io.BytesIO()
-    # We expect at least python 3.5 so protocol 4 is good.
-    _RestrictedPickler(file_obj, protocol=4, fix_imports=False).dump(obj)
+    _RestrictedPickler(file_obj, protocol=protocol, fix_imports=False).dump(obj)
     if not file_obj_passed:
         return file_obj.getvalue()
 
 
-def pickle_load(content, safe_to_import=None):
+def pickle_load(content=None, file_obj=None, safe_to_import=None):
     """
     **pickle_load**
     Load the pickled content. content should be a bytes object.
 
     **Parameters**
 
-    content : Bytes of pickled object. It needs to have Delta header in it that is
-        separated by a newline character from the rest of the pickled object.
+    content : Bytes of pickled object. 
+
+    file_obj : A file object to load the content from
 
     safe_to_import : A set of modules that needs to be explicitly allowed to be loaded.
         Example: {'mymodule.MyClass', 'decimal.Decimal'}
@@ -351,9 +374,13 @@ def pickle_load(content, safe_to_import=None):
 
 
     """
+    if not content and not file_obj:
+        raise ValueError('Please either pass the content or the file_obj to pickle_load.') 
     if isinstance(content, str):
         content = content.encode('utf-8')
-    return _RestrictedUnpickler(io.BytesIO(content), safe_to_import=safe_to_import).load()
+    if content:
+        file_obj = io.BytesIO(content)
+    return _RestrictedUnpickler(file_obj, safe_to_import=safe_to_import).load()
 
 
 def _get_pretty_form_text(verbose_level):
@@ -408,25 +435,27 @@ def load_path_content(path, file_type=None):
         file_type = path.split('.')[-1]
     if file_type == 'json':
         with open(path, 'r') as the_file:
-            content = json.load(the_file)
+            content = json_loads(the_file.read())
     elif file_type in {'yaml', 'yml'}:
         if yaml is None:  # pragma: no cover.
             raise ImportError('Pyyaml needs to be installed.')  # pragma: no cover.
         with open(path, 'r') as the_file:
             content = yaml.safe_load(the_file)
     elif file_type == 'toml':
-        if toml is None:  # pragma: no cover.
-            raise ImportError('Toml needs to be installed.')  # pragma: no cover.
-        with open(path, 'r') as the_file:
-            content = toml.load(the_file)
+        if tomli is None:  # pragma: no cover.
+            raise ImportError('On python<=3.10 tomli needs to be installed.')  # pragma: no cover.
+        with open(path, 'rb') as the_file:
+            content = tomli.load(the_file)
     elif file_type == 'pickle':
         with open(path, 'rb') as the_file:
             content = the_file.read()
             content = pickle_load(content)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv is None:  # pragma: no cover.
-            raise ImportError('CleverCSV needs to be installed.')  # pragma: no cover.
-        content = clevercsv.read_dicts(path)
+        if clevercsv:  # pragma: no cover.
+            content = clevercsv.read_dicts(path)
+        else:
+            with open(path, 'r') as the_file:
+                content = list(csv.DictReader(the_file))
         logger.info(f"NOTE: CSV content was empty in {path}")
 
         # Everything in csv is string but we try to automatically convert any numbers we find
@@ -470,26 +499,29 @@ def save_content_to_path(content, path, file_type=None, keep_backup=True):
 def _save_content(content, path, file_type, keep_backup=True):
     if file_type == 'json':
         with open(path, 'w') as the_file:
-            content = json.dump(content, the_file)
+            content = json_dumps(content)
+            the_file.write(content)
     elif file_type in {'yaml', 'yml'}:
         if yaml is None:  # pragma: no cover.
             raise ImportError('Pyyaml needs to be installed.')  # pragma: no cover.
         with open(path, 'w') as the_file:
             content = yaml.safe_dump(content, stream=the_file)
     elif file_type == 'toml':
-        if toml is None:  # pragma: no cover.
-            raise ImportError('Toml needs to be installed.')  # pragma: no cover.
-        with open(path, 'w') as the_file:
-            content = toml.dump(content, the_file)
+        if tomli_w is None:  # pragma: no cover.
+            raise ImportError('Tomli-w needs to be installed.')  # pragma: no cover.
+        with open(path, 'wb') as the_file:
+            content = tomli_w.dump(content, the_file)
     elif file_type == 'pickle':
         with open(path, 'wb') as the_file:
             content = pickle_dump(content, file_obj=the_file)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv is None:  # pragma: no cover.
-            raise ImportError('CleverCSV needs to be installed.')  # pragma: no cover.
+        if clevercsv:  # pragma: no cover.
+            dict_writer = clevercsv.DictWriter
+        else:
+            dict_writer = csv.DictWriter
         with open(path, 'w', newline='') as csvfile:
             fieldnames = list(content[0].keys())
-            writer = clevercsv.DictWriter(csvfile, fieldnames=fieldnames)
+            writer = dict_writer(csvfile, fieldnames=fieldnames)
             writer.writeheader()
             writer.writerows(content)
     else:
@@ -498,15 +530,30 @@ def _save_content(content, path, file_type, keep_backup=True):
     return content
 
 
+def _serialize_decimal(value):
+    if value.as_tuple().exponent == 0:
+        return int(value)
+    else:
+        return float(value)
+
+
 JSON_CONVERTOR = {
-    decimal.Decimal: float,
+    decimal.Decimal: _serialize_decimal,
     ordered_set.OrderedSet: list,
+    set: list,
     type: lambda x: x.__name__,
     bytes: lambda x: x.decode('utf-8'),
     datetime.datetime: lambda x: x.isoformat(),
     uuid.UUID: lambda x: str(x),
+    np_float32: float,
+    np_float64: float,
+    np_int32: int,
+    np_int64: int
 }
 
+if PydanticBaseModel:
+    JSON_CONVERTOR[PydanticBaseModel] = lambda x: x.dict()
+
 
 def json_convertor_default(default_mapping=None):
     if default_mapping:
@@ -541,12 +588,20 @@ class JSONDecoder(json.JSONDecoder):
 def json_dumps(item, default_mapping=None, **kwargs):
     """
     Dump json with extra details that are not normally json serializable
-
-    Note: I tried to replace json with orjson for its speed. It does work
-    but the output it makes is a byte object and Postgres couldn't directly use it without
-    encoding to str. So I switched back to json.
     """
-    return json.dumps(item, default=json_convertor_default(default_mapping=default_mapping), **kwargs)
+    if orjson:
+        indent = kwargs.pop('indent', None)
+        if indent:
+            kwargs['option'] = orjson.OPT_INDENT_2
+        return orjson.dumps(
+            item,
+            default=json_convertor_default(default_mapping=default_mapping),
+            **kwargs).decode(encoding='utf-8')
+    else:
+        return json.dumps(
+            item,
+            default=json_convertor_default(default_mapping=default_mapping),
+            **kwargs)
 
 
 json_loads = partial(json.loads, cls=JSONDecoder)
