@@ -2,13 +2,16 @@
 import datetime
 import pytest
 import logging
+import re
 import uuid
 from enum import Enum
+from typing import List
 from decimal import Decimal
 from deepdiff import DeepDiff
-from deepdiff.helper import pypy3
+from deepdiff.helper import pypy3, PydanticBaseModel
 from tests import CustomClass
 
+
 logging.disable(logging.CRITICAL)
 
 
@@ -294,6 +297,24 @@ class TestDeepDiffText:
         result = {}
         assert result == ddiff
 
+    def test_diff_quote_in_string(self):
+        t1 = {
+            "a']['b']['c": 1
+        }
+        t2 = {
+            "a']['b']['c": 2
+        }
+        diff = DeepDiff(t1, t2)
+        expected = {'values_changed': {'''root["a']['b']['c"]''': {'new_value': 2, 'old_value': 1}}}
+        assert expected == diff
+
+    def test_diff_quote_and_double_quote_in_string(self):
+        t1 = {'''a'"a''': 1}
+        t2 = {'''a'"a''': 2}
+        diff = DeepDiff(t1, t2)
+        expected = {'values_changed': {'root["a\'"a"]': {'new_value': 2, 'old_value': 1}}}
+        assert expected == diff
+
     def test_bytes(self):
         t1 = {
             1: 1,
@@ -426,45 +447,69 @@ class TestDeepDiffText:
         t1 = {1: 1, 2: 2, 3: 3, 4: {"a": "hello", "b": [1, 2, 5]}}
         t2 = {1: 1, 2: 2, 3: 3, 4: {"a": "hello", "b": [1, 3, 2, 5]}}
         ddiff = DeepDiff(t1, t2)
+        expected = {'iterable_item_added': {"root[4]['b'][1]": 3}}
+        assert expected == ddiff
+
+    def test_list_difference4(self):
+        t1 = ["a", "b", "d", "e"]
+        t2 = ["a", "b", "c", "d", "e"]
+        ddiff = DeepDiff(t1, t2)
+        result = {'iterable_item_added': {'root[2]': 'c'}}
+        assert result == ddiff
+
+    def test_list_difference5(self):
+        t1 = ["a", "b", "d", "e", "f", "g"]
+        t2 = ["a", "b", "c", "d", "e", "f"]
+        ddiff = DeepDiff(t1, t2)
+        result = {'iterable_item_added': {'root[2]': 'c'}, 'iterable_item_removed': {'root[5]': 'g'}}
+        assert result == ddiff
+
+    def test_list_difference_with_tiny_variations(self):
+        t1 = ['a', 'b', 'c', 'd']
+        t2 = ['f', 'b', 'a', 'g']
+
+        values = {
+            'a': 2.0000000000000027,
+            'b': 2.500000000000005,
+            'c': 2.000000000000002,
+            'd': 3.000000000000001,
+            'f': 2.000000000000003,
+            'g': 3.0000000000000027,
+        }
+        ddiff = DeepDiff(t1, t2)
         result = {
             'values_changed': {
-                "root[4]['b'][2]": {
-                    'new_value': 2,
-                    'old_value': 5
+                'root[0]': {
+                    'new_value': 'f',
+                    'old_value': 'a'
                 },
-                "root[4]['b'][1]": {
-                    'new_value': 3,
-                    'old_value': 2
+                'root[2]': {
+                    'new_value': 'a',
+                    'old_value': 'c'
+                },
+                'root[3]': {
+                    'new_value': 'g',
+                    'old_value': 'd'
                 }
-            },
-            'iterable_item_added': {
-                "root[4]['b'][3]": 5
             }
         }
         assert result == ddiff
 
-    def test_list_difference4(self):
-        # TODO: Look into Levenshtein algorithm
-        # So that the result is just insertion of "c" in this test.
-        t1 = ["a", "b", "d", "e"]
-        t2 = ["a", "b", "c", "d", "e"]
-        ddiff = DeepDiff(t1, t2)
-        result = {
-            'values_changed': {
-                'root[2]': {
-                    'new_value': 'c',
-                    'old_value': 'd'
-                },
-                'root[3]': {
-                    'new_value': 'd',
-                    'old_value': 'e'
-                }
-            },
-            'iterable_item_added': {
-                'root[4]': 'e'
+        ddiff2 = DeepDiff(t1, t2, zip_ordered_iterables=True)
+        assert result == ddiff2
+        # Now we change the characters with numbers with tiny variations
+
+        t3 = [2.0000000000000027, 2.500000000000005, 2.000000000000002, 3.000000000000001]
+        t4 = [2.000000000000003, 2.500000000000005, 2.0000000000000027, 3.0000000000000027]
+        ddiff3 = DeepDiff(t3, t4)
+
+        expected = {'values_changed': {}}
+        for path, report in result['values_changed'].items():
+            expected['values_changed'][path] = {
+                'new_value': values[report['new_value']],
+                'old_value': values[report['old_value']],
             }
-        }
-        assert result == ddiff
+        assert expected == ddiff3
 
     def test_list_of_booleans(self):
         t1 = [False, False, True, True]
@@ -579,6 +624,64 @@ class TestDeepDiffText:
         }
         assert ddiff == result
 
+    def test_precompiled_regex(self):
+
+        pattern_1 = re.compile('foo')
+        pattern_2 = re.compile('foo')
+        pattern_3 = re.compile('foo', flags=re.I)
+        pattern_4 = re.compile('(foo)')
+        pattern_5 = re.compile('bar')
+
+        # same object
+        ddiff = DeepDiff(pattern_1, pattern_1)
+        result = {}
+        assert ddiff == result
+
+        # same pattern, different object
+        ddiff = DeepDiff(pattern_1, pattern_2)
+        result = {}
+        assert ddiff == result
+
+        # same pattern, different flags
+        ddiff = DeepDiff(pattern_1, pattern_3)
+        result = {
+            'values_changed': {
+                'root.flags': {
+                    'new_value': 34,
+                    'old_value': 32,
+                },
+            }
+        }
+        assert ddiff == result
+
+        # same pattern, different groups
+        ddiff = DeepDiff(pattern_1, pattern_4)
+        result = {
+            'values_changed': {
+                'root.pattern': {
+                    'new_value': '(foo)',
+                    'old_value': 'foo',
+                },
+                'root.groups': {
+                    'new_value': 1,
+                    'old_value': 0,
+                },
+            }
+        }
+        assert ddiff == result
+
+        # different pattern
+        ddiff = DeepDiff(pattern_1, pattern_5)
+        result = {
+            'values_changed': {
+                'root.pattern': {
+                    'new_value': 'bar',
+                    'old_value': 'foo',
+                },
+            }
+        }
+        assert ddiff == result
+
     def test_custom_objects_change(self):
         t1 = CustomClass(1)
         t2 = CustomClass(2)
@@ -1059,18 +1162,19 @@ class TestDeepDiffText:
         }
         assert result == ddiff
 
-    @pytest.mark.parametrize("t1, t2, ignore_numeric_type_changes, significant_digits, number_format_notation, result", [
-        (43.265798602382986, 43.71677762295505, False, 0, "f", {'values_changed': {'root': {'new_value': 43.71677762295505, 'old_value': 43.265798602382986}}}),  # Note that it rounds the number so one becomes 43 and the other one is 44
-        (Decimal('2.5'), Decimal('1.5'), False, 0, "f", {}),
-        (Decimal('2.5'), Decimal('1.5'), False, 1, "f", {'values_changed': {'root': {'new_value': Decimal('1.5'), 'old_value': Decimal('2.5')}}}),
-        (Decimal('2.5'), Decimal(2.5), False, 3, "f", {}),
-        (1024, 1022, False, 2, "e", {}),
-        ({"key": [Decimal('2.0001'), Decimal('20000.0001')]}, {"key": [2.0002, 20000.0002]}, True, 4, "e", {'values_changed': {"root['key'][0]": {'new_value': 2.0002, 'old_value': Decimal('2.0001')}}})
+    @pytest.mark.parametrize("test_num, t1, t2, ignore_numeric_type_changes, significant_digits, number_format_notation, result", [
+        (1, 43.265798602382986, 43.71677762295505, False, 0, "f", {'values_changed': {'root': {'new_value': 43.71677762295505, 'old_value': 43.265798602382986}}}),  # Note that it rounds the number so one becomes 43 and the other one is 44
+        (2, Decimal('2.5'), Decimal('1.5'), False, 0, "f", {}),
+        (3, Decimal('2.5'), Decimal('1.5'), False, 1, "f", {'values_changed': {'root': {'new_value': Decimal('1.5'), 'old_value': Decimal('2.5')}}}),
+        (4, Decimal('2.5'), Decimal(2.5), False, 3, "f", {}),
+        (5, 1024, 1022, False, 2, "e", {}),
+        (6, {"key": [Decimal('2.0001'), Decimal('20000.0001')]}, {"key": [2.0002, 20000.0002]}, True, 4, "e", {'values_changed': {"root['key'][0]": {'new_value': 2.0002, 'old_value': Decimal('2.0001')}}}),
+        (7, [Decimal("999.99999999")], [Decimal("999.9999999999")], False, 6, "f", {}),
     ])
-    def test_significant_digits_and_notation(self, t1, t2, ignore_numeric_type_changes, significant_digits, number_format_notation, result):
+    def test_significant_digits_and_notation(self, test_num, t1, t2, ignore_numeric_type_changes, significant_digits, number_format_notation, result):
         ddiff = DeepDiff(t1, t2, significant_digits=significant_digits, number_format_notation=number_format_notation,
                          ignore_numeric_type_changes=ignore_numeric_type_changes)
-        assert result == ddiff
+        assert result == ddiff, f"test_significant_digits_and_notation #{test_num} failed."
 
     def test_significant_digits_for_complex_imaginary_part(self):
         t1 = 1.23 + 1.222254j
@@ -1226,6 +1330,10 @@ class TestDeepDiffText:
         result = {'values_changed': {'root[4]': {'new_value': 'now', 'old_value': now}}}
         assert result == ddiff
 
+    def test_ignore_type_in_groups_float_vs_decimal(self):
+        diff = DeepDiff(float('0.1'), Decimal('0.1'), ignore_type_in_groups=[(float, Decimal)], significant_digits=2)
+        assert not diff
+
     @pytest.mark.parametrize("t1, t2, significant_digits, result", [
         ([0.1], [Decimal('0.10')], 55,
             {'values_changed': {'root[0]': {'new_value': Decimal('0.10'), 'old_value': 0.1}}}),  # Due to floating point arithmetics with high significant digits.
@@ -1426,6 +1534,30 @@ class TestDeepDiffText:
         result = {}
         assert result == ddiff
 
+    def test_include_obj_callback(self):
+        def include_obj_callback(obj, path):
+            return True if "include" in path or isinstance(obj, int) else False
+
+        t1 = {"x": 10, "y": "b", "z": "c", "include_me": "a"}
+        t2 = {"x": 10, "y": "c", "z": "b", "include_me": "b"}
+        ddiff = DeepDiff(t1, t2, include_obj_callback=include_obj_callback)
+        result = {'values_changed': {"root['include_me']": {'new_value': "b", 'old_value': "a"}}}
+        assert result == ddiff
+        assert {"root['include_me']"} == ddiff.affected_paths
+        assert {"include_me"} == ddiff.affected_root_keys
+
+    def test_include_obj_callback_strict(self):
+        def include_obj_callback_strict(obj, path):
+            return True if isinstance(obj, int) and obj > 10 else False
+
+        t1 = {"x": 11, "y": 10, "z": "c"}
+        t2 = {"x": 12, "y": 12, "z": "c"}
+        ddiff = DeepDiff(t1, t2, include_obj_callback_strict=include_obj_callback_strict)
+        result = {'values_changed': {"root['x']": {'new_value': 12, 'old_value': 11}}}
+        assert result == ddiff
+        assert {"root['x']"} == ddiff.affected_paths
+        assert {"x"} == ddiff.affected_root_keys
+
     def test_skip_exclude_obj_callback(self):
         def exclude_obj_callback(obj, path):
             return True if "skip" in path or isinstance(obj, int) else False
@@ -1502,6 +1634,35 @@ class TestDeepDiffText:
         assert result == ddiff
         assert {"root[2]"} == ddiff.affected_paths
 
+    def test_list_item_removed_from_the_middle(self):
+        t1 = [0, 1, 2, 3, 'bye', 5, 6, 7, 8, 'a', 'b', 'c']
+        t2 = [0, 1, 2, 3, 5, 6, 7, 8, 'a', 'b', 'c']
+        diff = DeepDiff(t1, t2)
+        result = {'iterable_item_removed': {'root[4]': 'bye'}}
+        assert result == diff
+        assert {"root[4]"} == diff.affected_paths
+        assert {4} == diff.affected_root_keys
+
+    def test_list_item_values_replace_in_the_middle(self):
+        t1 = [0, 1, 2, 3, 'bye', 5, 6, 7, 8, 'a', 'b', 'c']
+        t2 = [0, 1, 2, 3, 'see', 'you', 'later', 5, 6, 7, 8, 'a', 'b', 'c']
+        diff = DeepDiff(t1, t2)
+        result = {
+            'values_changed': {
+                'root[4]': {
+                    'old_value': 'bye',
+                    'new_value': 'see',
+                }
+            },
+            'iterable_item_added': {
+                'root[5]': 'you',
+                'root[6]': 'later'
+            }
+        }
+        assert result == diff
+        assert {'root[5]', 'root[6]', 'root[4]'} == diff.affected_paths
+        assert {4, 5, 6} == diff.affected_root_keys
+
     def test_non_subscriptable_iterable(self):
         def gen1():
             yield 42
@@ -1568,6 +1729,79 @@ class TestDeepDiffText:
             'old_value': 'Blue'}}}
         assert expected_grouped == diff
 
+    def test_group_by2_when_repeats(self):
+        t1 = [
+            {'id': 'AA', 'name': 'Joe', 'last_name': 'Nobody', 'int_id': 2},
+            {'id': 'BB', 'name': 'James', 'last_name': 'Blue', 'int_id': 20},
+            {'id': 'BB', 'name': 'Jimmy', 'last_name': 'Red', 'int_id': 3},
+            {'id': 'CC', 'name': 'Mike', 'last_name': 'Apple', 'int_id': 4},
+        ]
+
+        t2 = [
+            {'id': 'AA', 'name': 'Joe', 'last_name': 'Nobody', 'int_id': 2},
+            {'id': 'BB', 'name': 'James', 'last_name': 'Brown', 'int_id': 20},
+            {'id': 'CC', 'name': 'Mike', 'last_name': 'Apple', 'int_id': 4},
+        ]
+
+        diff = DeepDiff(t1, t2, group_by='id', group_by_sort_key='name')
+        expected_grouped = {
+            'values_changed': {
+                "root['BB'][0]['last_name']": {
+                    'new_value': 'Brown',
+                    'old_value': 'Blue'
+                }
+            },
+            'iterable_item_removed': {
+                "root['BB'][1]": {
+                    'name': 'Jimmy',
+                    'last_name': 'Red',
+                    'int_id': 3
+                }
+            }
+        }
+        assert expected_grouped == diff
+
+        diff2 = DeepDiff(t1, t2, group_by='id', group_by_sort_key=lambda x: x['name'])
+        assert expected_grouped == diff2
+
+    def test_group_by3_when_repeats_and_group_by_list(self):
+        t1 = [
+            {'id': 'AA', 'name': 'Joe', 'last_name': 'Nobody', 'int_id': 2},
+            {'id': 'BB', 'name': 'James', 'last_name': 'Blue', 'int_id': 20},
+            {'id': 'BB', 'name': 'Jimmy', 'last_name': 'Red', 'int_id': 3},
+            {'id': 'CC', 'name': 'Mike', 'last_name': 'Apple', 'int_id': 4},
+        ]
+
+        t2 = [
+            {'id': 'AA', 'name': 'Joe', 'last_name': 'Nobody', 'int_id': 2},
+            {'id': 'BB', 'name': 'James', 'last_name': 'Brown', 'int_id': 20},
+            {'id': 'CC', 'name': 'Mike', 'last_name': 'Apple', 'int_id': 4},
+        ]
+
+        diff1 = DeepDiff(t1, t2, group_by=['id', 'name'])
+        expected_grouped = {
+            'dictionary_item_removed': ["root['BB']['Jimmy']"],
+            'values_changed': {
+                "root['BB']['James']['last_name']": {
+                    'new_value': 'Brown',
+                    'old_value': 'Blue'
+                }
+            }
+        }
+        assert expected_grouped == diff1
+
+        diff2 = DeepDiff(t1, t2, group_by=['id', 'name'], group_by_sort_key='int_id')
+        expected_grouped = {
+            'dictionary_item_removed': ["root['BB']['Jimmy']"],
+            'values_changed': {
+                "root['BB']['James'][0]['last_name']": {
+                    'new_value': 'Brown',
+                    'old_value': 'Blue'
+                }
+            }
+        }
+        assert expected_grouped == diff2
+
     def test_group_by_key_missing(self):
         t1 = [
             {'id': 'AA', 'name': 'Joe', 'last_name': 'Nobody'},
@@ -1714,3 +1948,63 @@ class TestDeepDiffText:
         }
 
         assert expected2 == diff2
+
+    def test_diffs_rrules(self):
+
+        from dateutil.rrule import MONTHLY, rrule
+
+        d = DeepDiff(
+            rrule(freq=MONTHLY, count=5, dtstart=datetime.datetime(2014, 12, 31)),
+            rrule(freq=MONTHLY, count=4, dtstart=datetime.datetime(2011, 12, 31)),
+        )
+
+        assert d == {
+            "values_changed": {
+                "root[0]": {
+                    "new_value": datetime.datetime(2011, 12, 31, 0, 0),
+                    "old_value": datetime.datetime(2014, 12, 31, 0, 0),
+                },
+                "root[1]": {
+                    "new_value": datetime.datetime(2012, 1, 31, 0, 0),
+                    "old_value": datetime.datetime(2015, 1, 31, 0, 0),
+                },
+                "root[2]": {
+                    "new_value": datetime.datetime(2012, 3, 31, 0, 0),
+                    "old_value": datetime.datetime(2015, 3, 31, 0, 0),
+                },
+                "root[3]": {
+                    "new_value": datetime.datetime(2012, 5, 31, 0, 0),
+                    "old_value": datetime.datetime(2015, 5, 31, 0, 0),
+                },
+            },
+            "iterable_item_removed": {"root[4]": datetime.datetime(2015, 7, 31, 0, 0)},
+        }
+
+    def test_pydantic1(self):
+
+        class Foo(PydanticBaseModel):
+            thing: int = None
+            that: str
+
+        t1 = Foo(thing=1, that='yes')
+        t2 = Foo(thing=2, that='yes')
+
+        diff = DeepDiff(t1, t2)
+        expected = {'values_changed': {'root.thing': {'new_value': 2, 'old_value': 1}}}
+        assert expected == diff
+
+    def test_pydantic2(self):
+
+        class Foo(PydanticBaseModel):
+            thing: int = None
+            that: str
+
+        class Bar(PydanticBaseModel):
+            stuff: List[Foo]
+
+        t1 = Bar(stuff=[Foo(thing=1, that='yes')])
+        t2 = Bar(stuff=[Foo(thing=2, that='yes')])
+
+        diff = DeepDiff(t1, t2)
+        expected = {'values_changed': {'root.stuff[0].thing': {'new_value': 2, 'old_value': 1}}}
+        assert expected == diff
